//+------------------------------------------------------------------+
//|                                                          ADR.mq4 |
//|                                     Copyright  2009, Apparelink |
//|                                        http://www.apparelink.com |
//+------------------------------------------------------------------+
/*REFERENCES
  http://www.aboutcurrency.com/university/currencypairs/most_volatile_active_traded_currency_pairs.shtml
*/

#property copyright "Copyright  2009, Apparelink"
#property link      "http://www.apparelink.com"

#include <stdlib.mqh>
#include <stderror.mqh>
#include <HelperFunctions.mqh>

#define MAX_IDX_COUNT 2

#property indicator_separate_window
#property indicator_maximum 500
#property indicator_level1 400
#property indicator_level2 300
#property indicator_level3 200
#property indicator_level4 100
#property indicator_minimum 0
#property indicator_levelwidth 1

#property indicator_buffers 2
#property indicator_color1 Blue
#property indicator_color2 Red

//---- default input parameters
extern int  FastPeriod = 12;
extern int  SlowPeriod = 26;
extern bool OptimizePerformance = true;
extern int  MaxBarCount = 400; //Limit the number of bars to reduce the calculation time

//---- global variables
string   g_short_name;
double   ExtMapBuffer1[];
double   ExtMapBuffer2[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   SetIndicatorBuffers();
   SetIndicatorDrawingStyle();
   CreateIndicatorShortName();
   CreateIndexToolTips();
   return (0);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
//----

//----
   return (0);
}
//+------------------------------------------------------------------+
//| Calculates the average daily range for the current symbol
//| and fills the index buffers with corresponding coefficients.
//+------------------------------------------------------------------+
int start()
{
   string symbols[];
   int timeframe = Period();
   FillIndexBuffer(Symbol(), timeframe, FastPeriod, ExtMapBuffer1);
   FillIndexBuffer(Symbol(), timeframe, SlowPeriod, ExtMapBuffer2);
}

//+------------------------------------------------------------------+
//| Sets the indicator drawing style for each line.
//+------------------------------------------------------------------+
void SetIndicatorDrawingStyle()
{
   SetIndexStyle(0, DRAW_LINE, STYLE_SOLID, 1);
   SetIndexDrawBegin(0, FastPeriod);
   SetIndexStyle(1, DRAW_LINE, STYLE_SOLID, 1);
   SetIndexDrawBegin(1, SlowPeriod);      
   IndicatorDigits(MAX_IDX_COUNT);
}

//+------------------------------------------------------------------+
//| Sets the index buffers used by the indicator.
//+------------------------------------------------------------------+
void SetIndicatorBuffers()
{
   IndicatorBuffers(MAX_IDX_COUNT);
   SetIndexBuffer(0, ExtMapBuffer1);
   SetIndexBuffer(1, ExtMapBuffer2);
}

//+------------------------------------------------------------------+
//| Creates indicator name for the DataWindow and indicator subwindow.
//|
//| Examples:
//| AR(12,26) 155.00 130.00
//| AR(20,50) 120.00 167.00
//+------------------------------------------------------------------+
void CreateIndicatorShortName()
{
   g_short_name = "AR(" + FastPeriod + "," + SlowPeriod + ")";
   IndicatorShortName(g_short_name);   
}

//+------------------------------------------------------------------+
//| Creates a tool tip on index line.
//+------------------------------------------------------------------+
void CreateIndexToolTips()
{
   SetIndexLabel(0, g_short_name + " Fast");
   SetIndexLabel(1, g_short_name + " Slow");
}

//+------------------------------------------------------------------+
//| Fills a numeric array (buffer) with iAR values.
//+------------------------------------------------------------------+
void FillIndexBuffer(string symbol, int timeframe, int period, double &array[])
{
   int current_bars = Bars,
       counted_bars = IndicatorCounted(),
       symbol_bars = iBars(symbol, timeframe),
       recalc_bars;

   if (counted_bars > 0)                              //Finished counting bars on current chart
      recalc_bars = current_bars - counted_bars;      //Recalculate only on the most recent bars
   else if (counted_bars == 0)                        //Still counting bars on current chart
   {
      if (OptimizePerformance == true)
         recalc_bars = MaxBarCount;
      else
         recalc_bars = symbol_bars;
   }
   else if (counted_bars < 0)                         //Count error
      return (-1);

   for (int shift = 0; shift < recalc_bars; shift++)  //Fill the array with iADR values
      array[shift] = iAR(symbol, timeframe, period, shift);
}

//+------------------------------------------------------------------+
//| Calculates the average range (AR) in pips using the past N
//| trading days.
//|
//| NOTE
//| AR is the same as ADR (average daily range) when the timeframe is
//| set to PERIOD_D1.  This function is written for code reusability.
//+------------------------------------------------------------------+
int iAR(string symbol, int timeframe, int period, int shift)
{
   double sum, limit = shift + period;
   for (; shift < limit; shift++)
      sum += ToPips(symbol, (iHigh(symbol, timeframe, shift) - iLow(symbol, timeframe, shift)));
   return (MathRound(sum/period));
}

//+------------------------------------------------------------------+
//| Converts a value into number of pips.
//+------------------------------------------------------------------+
int ToPips(string symbol, double value)
{
   if (StringLen(FindAndReplace(symbol,"JPY","")) < StringLen(symbol))  //The symbol has a Yen component
      return (value * 100);
   else                                                                 //The symbol does not have a Yen component
      return (value * 10000);
}
//+------------------------------------------------------------------+